@@ -149,12 +149,18 @@ class ConductorManager(object):
         # device is added
         for a in added:
             new_driver_dev_obj = new_driver_device_list[new_cpid_list.index(a)]
-            new_driver_dev_obj.create(context, host)
-            # TODO(All): If report data to Placement raise exception, we
-            # should revert driver device created in Cyborg and rp created
-            # in Placement to reduce the risk of data inconsistency here
-            # between Cyborg and Placement, we will consider to fix in V
-            # release.
+            try:
+                new_driver_dev_obj.create(context, host)
+            except Exception as exc:
+                LOG.exception("Failed to add device %(device)s. "
+                              "Reason: %(reason)s",
+                              {'device': new_driver_dev_obj,
+                               'reason': exc})
+                new_driver_dev_obj.destroy(context, host)
+            # TODO(All): If report device data to Placement raise exception,
+            # we should revert driver device created in Cyborg and resources
+            # created in Placement to reduce the risk of data inconsistency
+            # here between Cyborg and Placement.
             cleanup_inconsistency_resources = False
             for driver_dep_obj in new_driver_dev_obj.deployable_list:
                 try:
@@ -238,11 +244,10 @@ class ConductorManager(object):
                           'reason': exc})
                 new_driver_dep_obj.destroy(context, device_id)
                 rp_uuid = self.get_rp_uuid_from_obj(new_driver_dep_obj)
-                # TODO(All): If report data to Placement raise exception, we
-                # should revert driver deployable created in Cyborg and
-                # rp created in Placement to reduce the risk of data
-                # inconsistency here between Cyborg and Placement,
-                # we will consider to fix in V release.
+                # TODO(All): If report deployable data to Placement raise
+                # exception, we should revert driver deployable created in
+                # Cyborg and resources created in Placement to reduce the risk
+                # of data inconsistency here between Cyborg and Placement.
                 self._delete_provider_and_sub_providers(context, rp_uuid)
         for s in same:
             # get the driver_dep_obj, diff the driver_dep layer
@@ -274,7 +279,7 @@ class ConductorManager(object):
 
     def drv_attr_make_diff(self, context, dep_id, old_driver_attr_list,
                            new_driver_attr_list):
-        """Diff new dirver-side Attribute Object lists with the old one."""
+        """Diff new driver-side Attribute Object lists with the old one."""
         LOG.info("Start differing attributes.")
         dep_obj = Deployable.get_by_id(context, dep_id)
         driver_dep = DriverDeployable.get_by_name(context, dep_obj.name)
@@ -284,13 +289,12 @@ class ConductorManager(object):
         old_key_list = [driver_attr_obj.key for driver_attr_obj in
                         old_driver_attr_list]
         same = set(new_key_list) & set(old_key_list)
-        # key is same, diff the value.
         # key is deleted.
         deleted = set(old_key_list) - same
         for d in deleted:
             old_driver_attr_obj = old_driver_attr_list[old_key_list.index(d)]
             self.placement_client.delete_trait_by_name(
-                rp_uuid, old_driver_attr_obj.value)
+                context, rp_uuid, old_driver_attr_obj.value)
             old_driver_attr_obj.delete_by_key(context, dep_id, d)
         # key is added.
         added = set(new_key_list) - same
@@ -299,6 +303,7 @@ class ConductorManager(object):
             new_driver_attr_obj.create(context, dep_id)
             self.placement_client.add_traits_to_rp(
                 rp_uuid, [new_driver_attr_obj.value])
+        # key is same, diff the value.
         for s in same:
             # value is not same, update
             new_driver_attr_obj = new_driver_attr_list[new_key_list.index(s)]
@@ -310,14 +315,23 @@ class ConductorManager(object):
                 # Update traits here.
                 if new_driver_attr_obj.key.startswith("trait"):
                     self.placement_client.delete_trait_by_name(
-                        rp_uuid, old_driver_attr_obj.value)
+                        context, rp_uuid, old_driver_attr_obj.value)
                     self.placement_client.add_traits_to_rp(
                         rp_uuid, [new_driver_attr_obj.value])
+                # Update resource classes here.
+                if new_driver_attr_obj.key.startswith("rc"):
+                    self.placement_client.ensure_resource_classes(
+                        context, [new_driver_attr_obj.value])
+                    inv_data = _gen_resource_inventory(
+                        new_driver_attr_obj.value, dep_obj.num_accelerators)
+                    self.placement_client.update_inventory(rp_uuid, inv_data)
+                    self.placement_client.delete_rc_by_name(
+                        context, old_driver_attr_obj.value)
 
     @classmethod
     def drv_ah_make_diff(cls, context, dep_id, cpid_id, old_driver_ah_list,
                          new_driver_ah_list):
-        """Diff new dirver-side AttachHandle Object lists with the old one."""
+        """Diff new driver-side AttachHandle Object lists with the old one."""
         LOG.info("Start differing attach_handles.")
         new_info_list = [driver_ah_obj.attach_info for driver_ah_obj in
                          new_driver_ah_list]
